@@ -45,7 +45,7 @@ users_temp = {}
 users = {}
 summary_interval = {}
 summary_interval_temp = {}
-hashtag_changesets = {}
+processed_changesets = {}
 countries_changesets = {}
 whitelisted_users = []
 
@@ -74,7 +74,7 @@ def collect_changefile_stats(
     if length and osm_obj_nodes:
         try:
             len_feature = osmium.geom.haversine_distance(osm_obj_nodes)
-        except Exception as ex:
+        except:
             # print("WARNING: way  incomplete." % w.id)
             pass
 
@@ -128,16 +128,27 @@ def collect_changefile_stats(
     if hashtags or changeset_meta:
         users[user].setdefault("countries", [])
         users[user].setdefault("hashtags", [])
+        users[user].setdefault("editors", [])
 
-        try:
-            for ch in hashtag_changesets[changeset]["countries"]:
-                if ch not in users[user]["countries"]:
-                    users[user]["countries"].append(ch)
-            for ch in hashtag_changesets[changeset]["hashtags"]:
-                if ch not in users[user]["hashtags"]:
-                    users[user]["hashtags"].append(ch)
-        except Exception as ex:
-            pass
+        if changeset in processed_changesets:
+            try:
+                users[user]["countries"] += [
+                    ctry
+                    for ctry in processed_changesets[changeset]["countries"]
+                    if ctry not in users[user]["countries"]
+                ]
+                users[user]["hashtags"] += [
+                    hstg
+                    for hstg in processed_changesets[changeset]["hashtags"]
+                    if hstg not in users[user]["hashtags"]
+                ]
+                users[user]["editors"] += [
+                    editor
+                    for editor in processed_changesets[changeset]["editors"]
+                    if editor not in users[user]["editors"]
+                ]
+            except:
+                pass
 
     # osm element count
     users[user][osm_type][action] += 1
@@ -183,10 +194,18 @@ def collect_changefile_stats(
             users[user].setdefault(f"{t}_create_len", 0)
             if summary:
                 summary_interval[timestamp].setdefault(f"{t}_create_len", 0)
-            if t in tags and action != "modify" and action != "delete":
-                if summary:
-                    summary_interval[timestamp][f"{t}_create_len"] += round(len_feature)
-                users[user][f"{t}_create_len"] += round(len_feature)
+            if tags:
+                if (
+                    t in tags
+                    and action != "modify"
+                    and action != "delete"
+                    and len_feature > 0
+                ):
+                    if summary:
+                        summary_interval[timestamp][f"{t}_create_len"] += round(
+                            len_feature
+                        )
+                    users[user][f"{t}_create_len"] += round(len_feature)
 
 
 def calculate_stats(
@@ -194,10 +213,10 @@ def calculate_stats(
 ):
     if hashtags:  # intersect with changesets
         if (
-            len(hashtag_changesets) > 0 or len(whitelisted_users) > 0
+            len(processed_changesets) > 0 or len(whitelisted_users) > 0
         ):  # make sure there are changesets to intersect if not meaning hashtag changeset not found no need to go for changefiles
 
-            if changeset in hashtag_changesets.keys() or uname in whitelisted_users:
+            if changeset in processed_changesets or uname in whitelisted_users:
                 collect_changefile_stats(
                     user,
                     uname,
@@ -250,25 +269,37 @@ class ChangesetHandler(osmium.SimpleHandler):
                     run_hashtag_check_logic = True
 
         if run_hashtag_check_logic:
-            if c.id not in hashtag_changesets.keys():
-                hashtag_changesets[c.id] = {"hashtags": [], "countries": []}
-                # get bbox
-                bounds = str(c.bounds)
-                if "invalid" not in bounds:
-                    bbox_list = bounds.strip("()").split(" ")
-                    minx, miny = bbox_list[0].split("/")
-                    maxx, maxy = bbox_list[1].split("/")
-                    bbox = box(float(minx), float(miny), float(maxx), float(maxy))
-                    # Create a point for the centroid of the bounding box
-                    centroid = bbox.centroid
-                    intersected_rows = countries_df[countries_df.intersects(centroid)]
-                    hashtags_comment = re.findall(r"#[\w-]+", c.tags["comment"])
-                    for hash_tag in hashtags_comment:
-                        if hash_tag not in hashtag_changesets[c.id]["hashtags"]:
-                            hashtag_changesets[c.id]["hashtags"].append(hash_tag)
-                    for i, row in intersected_rows.iterrows():
-                        if row["name"] not in hashtag_changesets[c.id]["countries"]:
-                            hashtag_changesets[c.id]["countries"].append(row["name"])
+            processed_changesets.setdefault(
+                c.id,
+                {
+                    "hashtags": [],
+                    "countries": [],
+                    "editors": [],
+                },
+            )
+            if (
+                "created_by" in c.tags
+                and c.tags["created_by"] not in processed_changesets[c.id]["editors"]
+            ):
+                processed_changesets[c.id]["editors"].append(c.tags["created_by"])
+
+            hashtags_comment = re.findall(r"#[\w-]+", c.tags["comment"])
+            for hash_tag in hashtags_comment:
+                if hash_tag not in processed_changesets[c.id]["hashtags"]:
+                    processed_changesets[c.id]["hashtags"].append(hash_tag)
+            # get bbox
+            bounds = str(c.bounds)
+            if "invalid" not in bounds:
+                bbox_list = bounds.strip("()").split(" ")
+                minx, miny = bbox_list[0].split("/")
+                maxx, maxy = bbox_list[1].split("/")
+                bbox = box(float(minx), float(miny), float(maxx), float(maxy))
+                # Create a point for the centroid of the bounding box
+                centroid = bbox.centroid
+                intersected_rows = countries_df[countries_df.intersects(centroid)]
+                for i, row in intersected_rows.iterrows():
+                    if row["name"] not in processed_changesets[c.id]["countries"]:
+                        processed_changesets[c.id]["countries"].append(row["name"])
 
 
 class ChangefileHandler(osmium.SimpleHandler):
@@ -280,52 +311,43 @@ class ChangefileHandler(osmium.SimpleHandler):
             version = n.version
             if n.deleted:
                 version = 0
-            try:
-                calculate_stats(
-                    n.uid, n.user, n.changeset, version, n.tags, "nodes", n.timestamp
-                )
-            except Exception as ex:
-                print(f"Warning: {n.id} parse error")
-                print(ex)
+
+            calculate_stats(
+                n.uid, n.user, n.changeset, version, n.tags, "nodes", n.timestamp
+            )
 
     def way(self, w):
         if w.timestamp >= start_date_utc and w.timestamp < end_date_utc:
             version = w.version
             if w.deleted:
                 version = 0
-            try:
-                calculate_stats(
-                    w.uid,
-                    w.user,
-                    w.changeset,
-                    version,
-                    w.tags,
-                    "ways",
-                    w.timestamp,
-                    w.nodes if length else None,
-                )
-            except Exception as ex:
-                print(f"Warning: {w.id} parse error")
-                print(ex)
+
+            calculate_stats(
+                w.uid,
+                w.user,
+                w.changeset,
+                version,
+                w.tags,
+                "ways",
+                w.timestamp,
+                w.nodes if length else None,
+            )
 
     def relation(self, r):
         if r.timestamp >= start_date_utc and r.timestamp < end_date_utc:
             version = r.version
             if r.deleted:
                 version = 0
-            try:
-                calculate_stats(
-                    r.uid,
-                    r.user,
-                    r.changeset,
-                    version,
-                    r.tags,
-                    "relations",
-                    r.timestamp,
-                )
-            except Exception as ex:
-                print(f"Warning: {r.id} parse error")
-                print(ex)
+
+            calculate_stats(
+                r.uid,
+                r.user,
+                r.changeset,
+                version,
+                r.tags,
+                "relations",
+                r.timestamp,
+            )
 
 
 def process_changefiles(url):
@@ -336,6 +358,7 @@ def process_changefiles(url):
     file_path = get_file_path_from_url(url, "changefiles")
     # Open the .osc.gz file in read-only mode
     handler = ChangefileHandler()
+
     if length:
         handler.apply_file(file_path[:-3], locations=True)
     else:
@@ -431,6 +454,13 @@ def parse_args():
         default=False,
     )
 
+    parser.add_argument(
+        "--meta",
+        action="store_true",
+        help="Generates stats_metadata.json including sequence info , start_data end_date , Will be useful when running daily/weekly/monthly by service/cron",
+        default=False,
+    )
+
     parser.add_argument(
         "--rows",
         type=int,
@@ -536,9 +566,9 @@ def parse_args():
     )
 
     parser.add_argument(
-        "--exclude_date_in_name",
+        "--include_date_in_name",
         action="store_true",
-        help="By default from and to date will be added to filename , You can skip this behaviour with this option",
+        help="You can append stats date and time in filename using this option",
         default=False,
     )
 
@@ -725,11 +755,16 @@ def main():
     if (end_date - start_date).days > 1:
 
         if args.hashtags:
-            print(
-                "Warning : Replication for Changeset is minutely , To download more than day data it might take a while depending upon your internet speed, Use --force to ignore this warning"
-            )
             if not args.force:
+                print(
+                    "Warning : Replication for Changeset is minutely , To download more than day data it might take a while depending upon your internet speed, Use --force to ignore this warning"
+                )
                 sys.exit()
+        for url in args.url:
+            if "minute" in url and "geofabrik" not in url:
+                print(
+                    "Warning : To Process more than day data consider using daily/hourly replciation files to avoid downloading huge data . To use daily pass : --url day , for Hourly : --url hour "
+                )
     print(f"Supplied start_date: {start_date} and end_date: {end_date}")
 
     if args.hashtags or args.changeset:
@@ -898,9 +933,6 @@ def main():
 
         df = pd.json_normalize(list(users.values()))
 
-        if hashtags or changeset_meta:
-            df["countries"] = df["countries"].apply(lambda x: ",".join(map(str, x)))
-
         df = df.assign(
             changes=df["nodes.create"]
             + df["nodes.modify"]
@@ -932,16 +964,18 @@ def main():
             df = df.reindex(columns=cols)
 
         if hashtags or changeset_meta:
+            df["countries"] = df["countries"].apply(lambda x: ",".join(map(str, x)))
+            df["editors"] = df["editors"].apply(lambda x: ",".join(map(str, x)))
             df["hashtags"] = df["hashtags"].apply(lambda x: ",".join(map(str, x)))
             column_to_move = "hashtags"
             df = df.assign(**{column_to_move: df.pop(column_to_move)})
 
         start_date = in_local_timezone(start_date_utc, args.timezone)
         end_date = in_local_timezone(end_date_utc, args.timezone)
+        fname = args.name
+        if args.include_date_in_name:
+            fname = f"""{args.name}_{start_date.strftime('%Y-%m-%d-%H-%M-%S%z')}_{end_date.strftime('%Y-%m-%d-%H-%M-%S%z')}"""
 
-        fname = f"{args.name}_{start_date}_{end_date}"
-        if args.exclude_date_in_name:
-            fname = args.name
         if "image" in args.format:  ### image used for twitter tweet
             # Convert the DataFrame to an image
             df_img = df.head(25)
@@ -1122,21 +1156,21 @@ def main():
         command = " ".join(sys.argv)
         start_repl_ts = seq_to_timestamp(start_seq_url, args.timezone)
         end_repl_ts = seq_to_timestamp(end_seq_url, args.timezone)
-
-        with open(f"{args.name}_metadata.json", "w", encoding="utf-8") as file:
-            file.write(
-                json.dumps(
-                    {
-                        "command": str(command),
-                        "source": str(args.url),
-                        "start_date": str(start_date),
-                        "start_seq": f"{start_seq} = {start_repl_ts}",
-                        "end_date": str(end_date),
-                        "end_seq": f"{end_seq} = {end_repl_ts}",
-                    }
+        if args.meta:
+            with open(f"{args.name}_metadata.json", "w", encoding="utf-8") as file:
+                file.write(
+                    json.dumps(
+                        {
+                            "command": str(command),
+                            "source": str(args.url),
+                            "start_date": str(start_date),
+                            "start_seq": f"{start_seq} = {start_repl_ts}",
+                            "end_date": str(end_date),
+                            "end_seq": f"{end_seq} = {end_repl_ts}",
+                        }
+                    )
                 )
-            )
-        print("Metadata Created")
+            print("Metadata Created")
 
     else:
         print("No data Found")
